@@ -1,5 +1,5 @@
 /***********************************************************************
- **
+ **F
  **   calculator.cpp
  **
  **   This file is part of Cumulus.
@@ -7,12 +7,12 @@
  ************************************************************************
  **
  **   Copyright (c): 2002      by André Somers
- **                  2008-2010 by Axel Pauli
+ **                  2008-2011 by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: calculator.cpp 4685 2011-09-21 16:52:31Z axel $
+ **   $Id: calculator.cpp 4744 2011-12-24 19:46:17Z axel $
  **
  **   This class provides different calculator functionalities
  **
@@ -139,10 +139,11 @@ const Altitude& Calculator::getAltimeterAltitude()
       return lastAGLAltitude; // AGL
       break;
     case 3:
-    default:
       return lastAHLAltitude; // AHL
       break;
     }
+
+  return lastAltitude;
 }
 
 const AltitudeCollection& Calculator::getAltitudeCollection()
@@ -405,7 +406,7 @@ void Calculator::calcDistance( bool autoWpSwitch )
   // considered in manual mode to make testing possible.
   bool inside = false;
 
-  if ( lastSpeed.getKph() > 35 || GpsNmea::gps->getGpsStatus() != GpsNmea::validFix )
+  if ( moving() || GpsNmea::gps->getGpsStatus() != GpsNmea::validFix )
     {
       inside = task->checkSector( curDistance, lastPosition, selectedWp->taskPointIndex );
     }
@@ -1173,12 +1174,12 @@ void Calculator::determineFlightStatus()
 #define MINTURNANGDIFF 4 //see above
 
   /*
-    We define a maximum altitude drift to refine the detection of a
+    We define a maximum altitude average drift to refine the detection of a
     standstill. Because GPS altitudes tend to drift, we can't rely
     on the exact same altitude over any length of time. We need to
     allow for some drift.
   */
-#define MAXALTDRIFT 10 //see above
+#define MAXALTDRIFTAVG 2 //see above
 
   /*
     We define an analysis time frame. This time frame determines how
@@ -1188,60 +1189,57 @@ void Calculator::determineFlightStatus()
   */
 #define TIMEFRAME 10
 
-  if (samplelist.count() < 5)
-    return; //we need to have some samples in order to be able to analyze anything.
+  if( samplelist.count() < TIMEFRAME )
+    {
+      // We need to have some samples in order to be able to analyze anything.
+      return;
+    }
 
-  flightmode newFlightMode=unknown;
+  FlightMode newFlightMode = unknown;
 
-  //get headings from the last two samples
-  int lastHead=samplelist[0].vector.getAngleDeg();
-  int prevHead=samplelist[1].vector.getAngleDeg();
+  // get headings from the last two samples
+  int lastHead = samplelist[0].vector.getAngleDeg();
+  int prevHead = samplelist[1].vector.getAngleDeg();
 
-  //get the time difference between these samples
-  int timediff=samplelist[1].time.secsTo(samplelist[0].time);
+  // get the time difference between these samples
+  int timediff = samplelist[1].time.secsTo(samplelist[0].time);
 
-  if (timediff==0)
-    return; //if the time difference is 0, return (just to be sure). This will only cause problems...
-
-
-  //we are not doing a full analysis if we already have a flight mode. It suffices to check some
-  //basic criteria.
-  //lastFlightMode=unknown; //force complete evaluation for now.
-  //qDebug("Anglediff: %d",angleDiff(lastHead, prevHead));
+  if (timediff == 0)
+    {
+      // If the time difference is 0, return (just to be sure). This will only cause problems...
+      return;
+    }
 
+  // we are not doing a full analysis if we already have a flight mode. It suffices to check some
+  // basic criteria.
+  // lastFlightMode=unknown; //force complete evaluation for now.
+  // qDebug("Anglediff: %d",angleDiff(lastHead, prevHead));
   switch (lastFlightMode)
     {
-    case standstill: //we are not moving at all!
-      if (samplelist[0].position == samplelist[1].position)    //may be too ridged, GPS errors could cause problems here
+    case standstill: // we are not moving at all!
+
+      if ( (samplelist[0].position == samplelist[1].position ||
+          ( dist(&samplelist[0].position, &samplelist[1].position) / double(timediff) ) < 0.005) &&
+           lastSpeed.getMps() <= 0.5 )
         {
-          return; //no change in flight mode
+          // may be too ridged, GPS errors could cause problems here
+          return;
         }
       else
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
         }
       break;
 
-    case wave: //we are not moving at all, except vertically!  Needs lots of tweaking and testing...
-      if (samplelist[0].position == samplelist[1].position)    //may be too ridged, GPS errors could cause problems here
-        {
-          return; //no change in flight mslot_flightModeChangedode
-        }
-      else
-        {
-          newFlightMode=unknown;
-        }
-      break;
-
-    case cruising: //we are flying from point A to point B
+    case cruising: // we are flying from point A to point B
       if (abs(angleDiff(lastHead, _cruiseDirection)) <  MAXCRUISEANGDIFF &&
-          samplelist[0].vector.getSpeed().getMps()>5)
+          samplelist[0].vector.getSpeed().getMps() > 0.5 )
         {
-          return; //no change in flight mode
+          return;
         }
       else
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
           break;
         }
 
@@ -1249,7 +1247,7 @@ void Calculator::determineFlightStatus()
       //turning left means: the heading is decreasing
       if (angleDiff(prevHead, lastHead) > (-MINTURNANGDIFF * timediff))
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
           break;
         }
       else
@@ -1257,12 +1255,11 @@ void Calculator::determineFlightStatus()
           return; //no change in flight mode
         }
 
-
     case circlingR:
       //turning right means: the heading is increasing
       if (angleDiff(prevHead, lastHead) < (MINTURNANGDIFF * timediff))
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
           break;
         }
       else
@@ -1271,129 +1268,145 @@ void Calculator::determineFlightStatus()
         }
 
     default:
-      newFlightMode=unknown;
+      newFlightMode = unknown;
+      break;
     }
 
-  if (newFlightMode==unknown)
+  if ( newFlightMode == unknown )
     {
       //we need some real analysis
-      QTime tmp= samplelist[0].time.addSecs(-TIMEFRAME); //reference time
-      int ls=1;
+      QTime tmp = samplelist[0].time.addSecs(-TIMEFRAME); // reference time
+      int ls = 1;
 
       while ((samplelist[ls].time > tmp) && ( ls < samplelist.count()-1) )
-        ls++;
+        {
+          ls++;
+        }
+
       //ls now contains the index of the oldest sample we will use for this analysis.
       //Newer samples have lower indices!
 
       //initialize some values we will be needing...
-      bool mayBeL=true;     //this may be a left turn (that is, no big dir change to the right)
-      bool mayBeR=true;     //this may be a right turn (that is, no big dir change to the left)
-      int totalDirChange=0;  //total heading change. If cruising, this will be low, if turning, it will be high
-      int maxSpeed=0;        //maximum speed obtained in this set of samples
-      int aDiff=0;                 //difference in heading between two samples
-      int totalAltChange=0;  //total change in altitude (absolute)
-      int maxAltChange=0;  //maximum change of altitude between samples.
-      int altChange=0;
-      bool break_analysis=false; //flag to indicate we can stop further analysis.
-
+      bool mayBeL = true;     //this may be a left turn (that is, no big dir change to the right)
+      bool mayBeR = true;     //this may be a right turn (that is, no big dir change to the left)
+      int totalDirChange = 0; //total heading change. If cruising, this will be low, if turning, it will be high
+      int maxSpeed = 0;       //maximum speed obtained in this set of samples
+      int aDiff = 0;          //difference in heading between two samples
+      int totalAltChange = 0; //total change in altitude (absolute)
+      int maxAltChange = 0;   //maximum change of altitude between samples.
+      int altChange = 0;
+      bool break_analysis = false; //flag to indicate we can stop further analysis.
 
       //loop through the samples to get some basic data we can use to distinguish flight modes
-      for (int i=ls-1;i>=0;i--)
+      for (int i = ls-1; i >= 0; i--)
         {
+          // aDiff can be positive or negative according to the turn direction
           aDiff = angleDiff( samplelist[i+1].vector.getAngleDeg(), samplelist[i].vector.getAngleDeg() );
-          //qDebug("analysis: angle1=%d, angle2=%d, diff=%d",int(samplelist->at(i+1)->vector.getAngleDeg()),int(samplelist->at(i)->vector.getAngleDeg()), aDiff);
+          // qDebug("analysis: angle1=%d, angle2=%d, aDiff=%d",int(samplelist[i+1].vector.getAngleDeg()),int(samplelist[i].vector.getAngleDeg()), aDiff);
           //qDebug("analysis: position=(%d, %d)", samplelist->at(i)->position.x(),samplelist->at(i)->position.y() );
-          totalDirChange += abs(aDiff);
-          maxSpeed = qMax( maxSpeed, (int) rint(samplelist[i].vector.getSpeed().getKph()));
+
+          // That is the average value of the direction change in degree.
+          // can be positive or negation.
+          totalDirChange += aDiff;
+
+          maxSpeed = qMax( maxSpeed, (int) rint(samplelist[i].vector.getSpeed().getMps()));
           altChange = int(samplelist[i].altitude.getMeters() - samplelist[i+1].altitude.getMeters());
           totalAltChange += altChange;
-          maxAltChange = int(qMax(abs(altChange), maxAltChange));
+          maxAltChange = qMax(abs(altChange), maxAltChange);
 
-          if (aDiff >  MINTURNANGDIFF)
-            mayBeL=false;
-          if (aDiff < -MINTURNANGDIFF)
-            mayBeR=false;
-          //qDebug("  analysis: sample: %d, speed: %f",i,samplelist->at(i)->vector.getKph());
+          if ( aDiff > MINTURNANGDIFF )
+            {
+              mayBeL = false;
+            }
+          else if ( aDiff < -MINTURNANGDIFF )
+            {
+              mayBeR = false;
+            }
+
+          // qDebug("  analysis: sample: %d, speed: %f",i,samplelist->at(i)->vector.getKph());
         }
-      //qDebug ("analysis: aDiff=%d, totalDirChange=%d, maxSpeed=%d, totalAltChange=%d, maxAltChange=%d, maybeLeft=%d, maybeRight=%d",aDiff, totalDirChange, maxSpeed, totalAltChange, maxAltChange, int(mayBeL), int(mayBeR));
 
-      //try standstill. We are using a value > 0 because of possible GPS errors.
+      // qDebug ("analysis: aDiff=%d, absADiff=%d, totalDirChange=%d, maxSpeed=%d, totalAltChange=%d, maxAltChange=%d, maybeLeft=%d, maybeRight=%d", aDiff, abs(aDiff), totalDirChange, maxSpeed, totalAltChange, maxAltChange, int(mayBeL), int(mayBeR));
+
+      // try standstill. We are using a value > 0 because of possible GPS errors.
       /*
         The detection of stand stills may be extended further by checking if the altitude matches the terrain altitude. If not
         (or no where near), we can not assume a standstill. This is probably wave flying.
       */
-      if (maxSpeed<10)   // if we get under 10 kph for maximum speed, we may be standing still
+      if ( maxSpeed < 0.5 ) // if we get under 0.5m/s for maximum speed, we may be standing still
         {
-          if (abs(totalAltChange) * 2 <= MAXALTDRIFT && maxAltChange <= MAXALTDRIFT)    //check if we had any significant altitude changes
+          // check if we had any significant altitude changes
+          if ( (abs(totalAltChange) / ls) <= MAXALTDRIFTAVG )
             {
-              newFlightMode=standstill;
-              break_analysis=true;
-            }
-          else
-            {
-              newFlightMode=wave;
-              break_analysis=true;
+              newFlightMode = standstill;
+              break_analysis = true;
             }
         }
 
-      if (!break_analysis)
+      if ( !break_analysis )
         {
-          //get the time difference between the first and the last sample. This might not be the 20 secs we were planning to use at all!
-          timediff= samplelist[ls-1].time.secsTo(samplelist[0].time);
+          // Get the time difference between the first and the last sample.
+          // This might not be the 20 secs we were planning to use at all!
+          timediff = samplelist[ls-1].time.secsTo(samplelist[0].time);
 
-          //see if we might be cruising...
-          if (mayBeL && mayBeR)   //basicly, we have been going (almost) strait it seems...
+          // see if we might be cruising...
+          if (mayBeL && mayBeR)
             {
-              if (totalDirChange < 2 * timediff)
+              if (abs(totalDirChange) < 2 * timediff)
                 {
-                  //qDebug("analysis: distance=%f m, time difference=%d s",dist(&samplelist->at(ls-1)->position,&samplelist[0].position)*1000,timediff);
-                  if (dist(&samplelist[ls-1].position, &samplelist[0].position)*1000 > 5*timediff) //our average speed should be at least 5 m/s to qualify for cruising
-                    newFlightMode=cruising;
-                  _cruiseDirection=samplelist[0].vector.getAngleDeg();
+                  // our average speed should be at least 5 m/s to qualify for cruising
+                  newFlightMode = cruising;
+                  break_analysis = true;
+                  _cruiseDirection = samplelist[0].vector.getAngleDeg();
                   // qDebug("Cruise direction: %d.",_cruiseDirection);
                 }
-              break_analysis=true;
             }
         }
 
       if (!break_analysis)
         {
-          //So, we are not standing still, nor are we cruising. Circling then maybe?
-          if ((MINTURNANGDIFF * timediff ) < totalDirChange)   //if circling, we should have a minimum average turnrate of MINTURNANGDIFF degrees per second
+          // So, we are not standing still, nor are we cruising. Circling then maybe?
+          if ( abs(totalDirChange) > (MINTURNANGDIFF * timediff) )
             {
+              // if circling, we should have a minimum average turnrate of
+              // MINTURNANGDIFF degrees per second
               if (mayBeL && !mayBeR)
-                newFlightMode=circlingL;
+                {
+                  newFlightMode = circlingL;
+                }
+
               if (mayBeR && !mayBeL)
-                newFlightMode=circlingR;
-              //we have a new mode: we are circling! If neither of the above are true, we are doing something
-              //funny, and the flight mode 'unknown' still applies, so we may also quit analyzing...
+                {
+                  newFlightMode = circlingR;
+                }
+
+              // we have a new mode: we are circling! If neither of the above are true, we are doing something
+              // funny, and the flight mode 'unknown' still applies, so we may also quit analyzing...
             }
         }
     }
 
-  if (newFlightMode!=lastFlightMode)
+  if (newFlightMode != lastFlightMode)
     {
-      lastFlightMode=newFlightMode;
-      samplelist[0].marker=++_marker;
-      // qDebug("new flightmode: %d",lastFlightMode);
-      slot_flightModeChanged(newFlightMode);
+      lastFlightMode = newFlightMode;
+      samplelist[0].marker = ++_marker;
+      // qDebug("new FlightMode: %d",lastFlightMode);
+      flightModeChanged( newFlightMode );
     }
 }
 
-
 /** Called if the status of the GPS changes. */
 void Calculator::slot_GpsStatus(GpsNmea::GpsStatus newState)
 {
   // qDebug("connection status changed...");
-  flightmode newFlightMode=unknown;
-  // qDebug("new flightmode=%d, last flightmode=%d",newFlightMode, lastFlightMode);
+  FlightMode newFlightMode = unknown;
 
-  if (newFlightMode!=lastFlightMode)
+  if (newFlightMode != lastFlightMode)
     {
-      lastFlightMode=newFlightMode;
+      lastFlightMode = newFlightMode;
       samplelist[0].marker=++_marker;
-      // qDebug("new flightmode: %d",lastFlightMode);
-      slot_flightModeChanged(newFlightMode);
+      // qDebug("new FlightMode: %d",lastFlightMode);
+      flightModeChanged(newFlightMode);
     }
 
   if ( newState == GpsNmea::noFix )
@@ -1405,9 +1418,8 @@ void Calculator::slot_GpsStatus(GpsNmea::GpsStatus newState)
     }
 }
 
-
-/** This slot is used internally to re-emit the flight mode signal with the marker value */
-void Calculator::slot_flightModeChanged(Calculator::flightmode fm)
+/** This function is used internally to emit the flight mode signal with the marker value */
+void Calculator::flightModeChanded(Calculator::FlightMode fm)
 {
   if ( _calculateWind )
     {
@@ -1489,8 +1501,6 @@ bool Calculator::matchesFlightMode(GeneralConfig::UseInMode mode)
     return lastFlightMode == standstill;
   if (mode==GeneralConfig::cruising)
     return lastFlightMode == cruising;
-  if (mode==GeneralConfig::wave)
-    return lastFlightMode == wave;
   if (mode==GeneralConfig::circling)
     return (lastFlightMode == circlingL || lastFlightMode == circlingR);
 
@@ -1628,3 +1638,32 @@ void Calculator::slot_startTask()
 
     slot_WaypointChange( tp2Taken, true );
 }
+
+/**
+ * @returns true if we are faster in move as or equal 35km/h.
+ */
+bool Calculator::moving()
+{
+  const double Limit = 35000.0 / 3600.0; // 35Km/h as m/s
+
+  if( samplelist.count() < 5 )
+    {
+      // We need to have some samples in order to be able to analyze speed.
+      return false;
+    }
+
+  double speed = 0.0;
+
+  for( int i = 0; i < 5; i++ )
+    {
+      // average speed about 5s
+      speed += samplelist[i].vector.getSpeed().getMps();
+    }
+
+  if( (speed / 5.0) >= Limit )
+    {
+      return true;
+    }
+
+  return false;
+};
