@@ -7,12 +7,12 @@
 ************************************************************************
 **
 **   Copyright (c):  2002      by Heiner Lamprecht
-**                   2007-2010 by Axel Pauli
+**                   2007-2011 by Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: flighttask.cpp 4597 2011-04-28 18:55:45Z axel $
+**   $Id: flighttask.cpp 4718 2011-12-08 19:24:23Z axel $
 **
 **   This class provides calculation, drawing and other support
 **   for flight tasks and contains the data of a task.
@@ -628,8 +628,8 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
   const int h = Map::getInstance()->size().height();
 
   // Set pen color and width for the course line
-  QColor courseLineColor = conf->getTaskCourseLineColor();
-  qreal courseLineWidth  = conf->getTaskCourseLineWidth();
+  QColor courseLineColor = conf->getTargetLineColor();
+  qreal courseLineWidth  = conf->getTargetLineWidth();
 
   // qDebug("QDesktop: w=%d, h=%d, ora=%d", w, h, ora );
 
@@ -645,6 +645,18 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
           drawnTp.append( tpList->at(loop) );
         }
 
+      // map projected point to map
+      QPoint mPoint(glMapMatrix->map(tpList->at(loop)->projP));
+
+      bool mPointIsContained = viewport.contains(mPoint);
+
+      if( mPointIsContained )
+        {
+          painter->setPen(QPen(Qt::black));
+          painter->setBrush( QBrush( Qt::black, Qt::SolidPattern ) );
+          painter->drawRect( mPoint.x()-5, mPoint.y()-5, 10, 10 );
+        }
+
       if( flightType == Unknown )
         {
           if( loop )
@@ -652,13 +664,10 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
               painter->setPen(QPen(courseLineColor, courseLineWidth));
               // Draws the course line
               painter->drawLine( glMapMatrix->map(tpList->at(loop - 1)->projP),
-               glMapMatrix->map(tpList->at(loop)->projP) );
+                                 glMapMatrix->map(tpList->at(loop)->projP) );
             }
         }
 
-      // map projected point to map
-      QPoint mPoint(glMapMatrix->map(tpList->at(loop)->projP));
-
       // convert biangle (90...180) from radian to degrees
       int biangle = (int) rint( ((tpList->at(loop)->angle) / M_PI ) * 180.0 );
 
@@ -666,7 +675,7 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
       {
       case TaskPointTypes::RouteP:
 
-        if( viewport.contains(mPoint) )
+        if( mPointIsContained )
           {
             QColor c;
 
@@ -689,14 +698,14 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
           {
             painter->setPen(QPen(courseLineColor, courseLineWidth));
             painter->drawLine( glMapMatrix->map(tpList->at(loop - 1)->projP),
-             glMapMatrix->map(tpList->at(loop)->projP) );
+                               glMapMatrix->map(tpList->at(loop)->projP) );
           }
 
         break;
 
       case TaskPointTypes::Begin:
 
-        if( viewport.contains(mPoint) )
+        if( mPointIsContained )
           {
             QColor c;
 
@@ -721,14 +730,14 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
           {
             painter->setPen(QPen(courseLineColor, courseLineWidth));
             painter->drawLine( glMapMatrix->map(tpList->at(loop - 1)->projP),
-             glMapMatrix->map(tpList->at(loop)->projP) );
+                               glMapMatrix->map(tpList->at(loop)->projP) );
           }
 
         break;
 
       case TaskPointTypes::End:
 
-        if( viewport.contains(mPoint) )
+        if( mPointIsContained )
           {
             QColor c;
 
@@ -761,7 +770,7 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
           {
             painter->setPen(QPen(courseLineColor, courseLineWidth));
             painter->drawLine( glMapMatrix->map(tpList->at(loop - 1)->projP),
-             glMapMatrix->map(tpList->at(loop)->projP ) );
+                               glMapMatrix->map(tpList->at(loop)->projP ) );
           }
 
         break;
@@ -776,16 +785,6 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
  */
 void FlightTask::circleSchemeDrawing( QPainter* painter, QList<Waypoint*> &drawnTp )
 {
-  extern MapConfig* _globalMapConfig;
-
-  // determine icon size to be used
-  int iconSize = 32;
-
-  if ( _globalMapConfig->useSmallIcons() )
-    {
-      iconSize = 16;
-    }
-
   // get user defined scheme items
   GeneralConfig *conf = GeneralConfig::instance();
 
@@ -806,8 +805,8 @@ void FlightTask::circleSchemeDrawing( QPainter* painter, QList<Waypoint*> &drawn
   const int h = Map::getInstance()->size().height();
 
   // Set pen color and width for the course line
-  QColor courseLineColor = conf->getTaskCourseLineColor();
-  qreal courseLineWidth  = conf->getTaskCourseLineWidth();
+  QColor courseLineColor = conf->getTargetLineColor();
+  qreal courseLineWidth  = conf->getTargetLineWidth();
 
   QRect viewport( -10-r, -10-r, w+2*r, h+2*r );
 
@@ -819,6 +818,17 @@ void FlightTask::circleSchemeDrawing( QPainter* painter, QList<Waypoint*> &drawn
           drawnTp.append( tpList->at(loop) );
         }
 
+      QPoint mPoint (glMapMatrix->map(tpList->at(loop)->projP));
+
+      bool mPointIsContained = viewport.contains(mPoint);
+
+      if( mPointIsContained )
+        {
+          painter->setPen(QPen(Qt::black));
+          painter->setBrush( QBrush( Qt::black, Qt::SolidPattern ) );
+          painter->drawRect( mPoint.x()-5, mPoint.y()-5, 10, 10 );
+        }
+
       if( loop )
         {
           // Draws the course line
@@ -834,11 +844,9 @@ void FlightTask::circleSchemeDrawing( QPainter* painter, QList<Waypoint*> &drawn
           continue;
         }
 
-      QPoint mPoint (glMapMatrix->map(tpList->at(loop)->projP));
-
       // qDebug("MappedPoint: x=%d, y=%d", mPoint.x(), mPoint.y() );
 
-      if( ! viewport.contains(mPoint) )
+      if( ! mPointIsContained )
         {
           // ignore not visible points
           continue;
