@@ -6,7 +6,7 @@
  ported to Qt4.x/X11  : (C) 2007-2011 by Axel Pauli
  maintainer           : axel@kflog.org
 
- $Id: mainwindow.cpp 4594 2011-04-06 19:16:46Z axel $
+ $Id: mainwindow.cpp 4741 2011-12-23 14:28:14Z axel $
 
 ****************************************************************************/
 
@@ -24,10 +24,10 @@
  *  and to initiate the load of the map and all other data.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <unistd.h>
-#include <string.h>
+#include <cstring>
 #include <signal.h>
 #include <sys/ioctl.h>
 
@@ -224,6 +224,7 @@ MainWindow::MainWindow( Qt::WindowFlags flags ) : QMainWindow( 0, flags )
   char *ldpath = getenv( "LD_LIBRARY_PATH" );
   char *display = getenv( "DISPLAY" );
   char *proxy = getenv( "http_proxy" );
+  char *Proxy = getenv( "HTTP_PROXY" );
 
   qDebug( "PWD=%s", pwd ? pwd : "NULL" );
   qDebug( "USER=%s", user ? user : "NULL" );
@@ -232,7 +233,8 @@ MainWindow::MainWindow( Qt::WindowFlags flags ) : QMainWindow( 0, flags )
   qDebug( "LD_LIBRARY_PATH=%s", ldpath ? ldpath : "NULL" );
   qDebug( "QDir::homePath()=%s", QDir::homePath().toLatin1().data() );
   qDebug( "DISPLAY=%s", display ? display : "NULL" );
-  qDebug( "HTTP_PROXY=%s", proxy ? proxy : "NULL" );
+  qDebug( "http_proxy=%s", proxy ? proxy : "NULL" );
+  qDebug( "HTTP_PROXY=%s", Proxy ? Proxy : "NULL" );
 
   qDebug( "UserDataDir=%s",
           GeneralConfig::instance()->getUserDataDirectory().toLatin1().data() );
@@ -503,6 +505,9 @@ void MainWindow::slotCreateApplicationWidgets()
            this, SLOT( slotSwitchToInfoView( Waypoint* ) ) );
   connect( viewMap->_theMap, SIGNAL( airspaceWarning( const QString&, const bool ) ),
            this, SLOT( slotAlarm( const QString&, const bool ) ) );
+  connect( viewMap->_theMap, SIGNAL( newPosition( QPoint& ) ),
+           calculator, SLOT( slot_changePosition( QPoint& ) ) );
+
   connect( viewMap, SIGNAL( toggleLDCalculation( const bool ) ),
            calculator, SLOT( slot_toggleLDCalculation(const bool) ) );
   connect( viewMap, SIGNAL( toggleMenu() ), this, SLOT( slotToggleMenu() ) );
@@ -559,8 +564,8 @@ void MainWindow::slotCreateApplicationWidgets()
            viewMap, SLOT( slot_LD( const double&, const double&) ) );
   connect( calculator, SIGNAL( newGlider( const QString&) ),
            viewMap, SLOT( slot_glider( const QString&) ) );
-  connect( calculator, SIGNAL( flightModeChanged( Calculator::flightmode ) ),
-           viewMap, SLOT( slot_setFlightStatus() ) );
+  connect( calculator, SIGNAL( flightModeChanged(Calculator::FlightMode) ),
+           viewMap, SLOT( slot_setFlightStatus(Calculator::FlightMode) ) );
 
   connect( calculator, SIGNAL( taskpointSectorTouched() ),
            logger, SLOT( slotTaskSectorTouched() ) );
@@ -568,6 +573,8 @@ void MainWindow::slotCreateApplicationWidgets()
            this, SLOT( slotNotification( const QString&, const bool ) ) );
   connect( calculator, SIGNAL( newSample() ),
            logger, SLOT( slotMakeFixEntry() ) );
+  connect( calculator, SIGNAL( flightModeChanged(Calculator::FlightMode) ),
+           logger, SLOT( slotFlightModeChanged(Calculator::FlightMode) ) );
 
   connect( ( QObject* ) calculator->getReachList(), SIGNAL( newReachList() ),
            this, SLOT( slotNewReachList() ) );
@@ -1821,6 +1828,22 @@ void MainWindow::slotVersion()
           "<br></html>" ));
 
   aw->setTeamText( team );
+
+  QString disclaimer( tr(
+            "<hml>"
+            "This program comes with"
+            "<p><b>ABSOLUTELY NO WARRANTY!</b></p>"
+            "Do not rely on this software program as your "
+            "primary source of navigation. You as user are "
+            "responsible for using official aeronautical "
+            "charts and proper methods for safe navigation. "
+            "The information presented by this application "
+            "may be outdated or incorrect."
+            "<p><b>Don't use this program if you cannot accept the disclaimer!</b></p>"
+            "</html>" ));
+
+  aw->setDisclaimerText( disclaimer );
+
   aw->resize( size() );
   aw->setVisible( true );
 }
@@ -1875,7 +1898,7 @@ void MainWindow::slotRememberWaypoint()
   wp.length = 0;
   AltitudeCollection alt = calculator->getAltitudeCollection();
   wp.elevation = int ( ( alt.gpsAltitude - alt.gndAltitude ).getMeters() );
-  wp.type = BaseMapElement::Turnpoint;
+  wp.type = BaseMapElement::Landmark;
   wp.isLandable = false;
   wp.country = GeneralConfig::instance()->getHomeCountryCode();
 
@@ -1992,7 +2015,7 @@ void MainWindow::slotReadconfig()
 /** Called if the status of the GPS changes, and controls the availability of manual navigation. */
 void MainWindow::slotGpsStatus( GpsNmea::GpsStatus status )
 {
-  playSound("notify");
+  static bool onePlay = false;
 
   if ( ( status != GpsNmea::validFix || calculator->isManualInFlight()) && ( view == mapView ) )
     {  // no GPS data
@@ -2001,6 +2024,16 @@ void MainWindow::slotGpsStatus( GpsNmea::GpsStatus status )
     }
   else
     {  // GPS data valid
+      if( onePlay == false )
+        {
+          // Only the first connect is reported via sound after the startup.
+          // That shall prevent multiple notifications if the GPS receiver
+          // plays ping pong (have a fix, have not a fix...) That can be very
+          // annoying. The current GPS state is signaled optical now.
+          playSound("notify");
+          onePlay = true;
+        }
+
       toggleManualNavActions( false );
       toggleGpsNavActions( true );
     }
@@ -2028,7 +2061,6 @@ void MainWindow::slotCenterToWaypoint()
     }
 }
 
-
 /** Called if the user pressed V in mapview.
   * Adjusts the zoom factor so that the currently selected waypoint
   * is displayed as good as possible. */
@@ -2151,6 +2183,8 @@ void MainWindow::slotNavigate2Home()
   wp.name = tr("Home");
   wp.description = tr("Home Site");
   wp.origP.setPos( GeneralConfig::instance()->getHomeCoord() );
+  wp.elevation = GeneralConfig::instance()->getHomeElevation().getMeters();
+  wp.country = GeneralConfig::instance()->getHomeCountryCode();
 
   calculator->slot_WaypointChange( &wp, true );
 }
