@@ -13,7 +13,7 @@
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: map.cpp 4687 2011-09-23 15:52:34Z axel $
+ **   $Id: map.cpp 4741 2011-12-23 14:28:14Z axel $
  **
  ***********************************************************************/
 
@@ -52,7 +52,6 @@
 #include "flarmdisplay.h"
 #endif
 
-extern MainWindow  *_globalMainWindow;
 extern MapContents *_globalMapContents;
 extern MapMatrix   *_globalMapMatrix;
 extern MapConfig   *_globalMapConfig;
@@ -79,6 +78,7 @@ Map::Map(QWidget* parent) : QWidget(parent)
   curMapRot = 0;
   heading = 0;
   bearing = 0;
+  lastRelBearing = -999;
   mode = northUp;
   m_scheduledFromLayer = baseLayer;
   ShowGlider = false;
@@ -492,6 +492,29 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
     }
 }
 
+void Map::wheelEvent(QWheelEvent *event)
+{
+  int numDegrees = event->delta() / 8;
+  int numSteps = numDegrees / 15;
+
+  if( abs(numSteps) > 10 || numSteps == 0 )
+    {
+      event->ignore();
+      return;
+    }
+
+  if( numSteps > 0 )
+    {
+      slotZoomIn();
+    }
+  else
+    {
+      slotZoomOut();
+    }
+
+  event->accept();
+}
+
 void Map::mousePressEvent(QMouseEvent* event)
 {
   // qDebug() << "Map::mousePressEvent(): Pos=" << event->pos();
@@ -804,6 +827,7 @@ void Map::__drawGrid()
       break;
     default:
       gridStep = 4;
+      break;
     }
 
   QPoint cP, cP2;
@@ -1112,7 +1136,13 @@ void Map::__redrawMap(mapLayer fromLayer, bool queueRequest)
  */
 void Map::__drawBaseLayer()
 {
-  if( !_isEnable ) return;
+  if( !_isEnable )
+    {
+      return;
+    }
+
+  m_drawnCityList.clear();
+  QList<BaseMapElement *> drawnElements;
 
   // Erase the base layer and fill it with the subterrain color. If there
   // are no terrain map data available, this is the default map ground color.
@@ -1145,37 +1175,43 @@ void Map::__drawBaseLayer()
   GpsNmea::gps->readDataFromGps();
 
   // next, draw the topographical elements and the cities
-  _globalMapContents->drawList(&baseMapP, MapContents::TopoList);
-  _globalMapContents->drawList(&baseMapP, MapContents::CityList);
-  _globalMapContents->drawList(&baseMapP, MapContents::LakeList);
+  _globalMapContents->drawList(&baseMapP, MapContents::TopoList, drawnElements);
+  _globalMapContents->drawList(&baseMapP, MapContents::CityList, m_drawnCityList);
+  _globalMapContents->drawList(&baseMapP, MapContents::LakeList, drawnElements);
 
   // draw the roads and the railroads
   if( cs <= 200.0 )
     {
-      _globalMapContents->drawList(&baseMapP, MapContents::RoadList);
-      _globalMapContents->drawList(&baseMapP, MapContents::RailList);
+      _globalMapContents->drawList(&baseMapP, MapContents::RoadList, drawnElements);
+      _globalMapContents->drawList(&baseMapP, MapContents::RailList, drawnElements);
     }
 
   // draw the hydro
   if( cs <= 500.0 )
-  {
-    _globalMapContents->drawList(&baseMapP, MapContents::HydroList);
- }
+    {
+      _globalMapContents->drawList(&baseMapP, MapContents::HydroList, drawnElements);
+    }
 
   // draw the landmarks and the obstacles
   if( cs < 1024.0 )
     {
-      _globalMapContents->drawList(&baseMapP, MapContents::LandmarkList);
-      _globalMapContents->drawList(&baseMapP, MapContents::ObstacleList);
-      _globalMapContents->drawList(&baseMapP, MapContents::ReportList);
+      _globalMapContents->drawList(&baseMapP, MapContents::LandmarkList, drawnElements);
+      _globalMapContents->drawList(&baseMapP, MapContents::ObstacleList, drawnElements);
+      _globalMapContents->drawList(&baseMapP, MapContents::ReportList, drawnElements);
     }
 
-  // draw the highways
-  _globalMapContents->drawList(&baseMapP, MapContents::HighwayList);
-  _globalMapContents->drawList(&baseMapP, MapContents::RadioList);
+  // draw the motorways
+  _globalMapContents->drawList(&baseMapP, MapContents::MotorwayList, drawnElements);
+  _globalMapContents->drawList(&baseMapP, MapContents::RadioList, drawnElements);
 
   // end the painter
   baseMapP.end();
+
+  // draw the city labels if scale is not to high
+  if( cs <= 100.0 )
+    {
+      _drawCityLabels( m_pixBaseMap );
+    }
 }
 
 /**
@@ -1672,6 +1708,48 @@ void Map::__drawLabel( QPainter* painter,
   painter->restore();
 }
 
+void Map::_drawCityLabels( QPixmap& pixmap )
+{
+  if( m_drawnCityList.size() == 0 )
+    {
+      return;
+    }
+
+  QString labelText;
+
+  QPainter painter(&pixmap);
+  QFont font = painter.font();
+#ifdef MAEMO
+  font.setPointSize( 10 );
+#else
+  font.setPointSize( 6 );
+#endif
+  painter.setFont( font );
+  painter.setBrush(Qt::NoBrush);
+  painter.setPen(QPen(Qt::black, 2, Qt::SolidLine));
+
+  QSet<QString> set;
+
+  for( int i = 0; i < m_drawnCityList.size(); i++ )
+    {
+      LineElement* city = static_cast<LineElement *> (m_drawnCityList.at(i));
+
+      // Get the screen bounding box of the city
+      QRect sbRect = city->getScreenBoundingBox();
+
+      // A city can consist of several segments at a border edge but we want to
+      // draw the name only once.
+      if( sbRect.isValid() && set.contains( city->getName() ) == false )
+        {
+          painter.drawText( sbRect.x() + sbRect.width() / 2,
+                            sbRect.y() + sbRect.height() / 2 + 3,
+                            city->getName() );
+
+          set.insert( city->getName() );
+        }
+    }
+}
+
 /** This function sets the map rotation and redraws the map if the
     map rotation differs too much from the current map rotation. */
 void Map::setMapRot(int newRotation)
@@ -2355,6 +2433,11 @@ void Map::__drawGlider()
   // now, draw the line from the glider symbol to the waypoint
   __drawDirectionLine(QPoint(Rx,Ry));
 
+  // draws a line from the current position into the movement direction.
+  __drawTrackLine( QPoint(Rx,Ry) );
+
+  __drawRelBearingInfo();
+
   // @ee the glider pixmap contains all rotated glider symbols.
   QPainter p(&m_pixInformationMap);
   p.drawPixmap( Rx-40, Ry-40, _glider, rot*80, 0, 80, 80 );
@@ -2392,7 +2475,7 @@ void Map::__drawX()
 }
 
 
-/** Used to zoom in on the map. Will schedule a redraw. */
+/** Used to zoom into the map. Will schedule a redraw. */
 void Map::slotZoomIn()
 {
   // @AP: block zoom events, if map is redrawn
@@ -2423,8 +2506,13 @@ void Map::slotZoomIn()
 
   zoomFactor /= zoomProgressiveVal[zoomProgressive];
 
+  if( zoomFactor < GeneralConfig::instance()->getMapLowerLimit() )
+    {
+      zoomFactor = GeneralConfig::instance()->getMapLowerLimit();
+    }
+
   scheduleRedraw();
-  QString msg = QString(tr("Map zoom in by factor %1")).arg(zoomFactor, 0, 'f', 1);
+  QString msg = QString(tr("Zoom scale 1:%1")).arg(zoomFactor, 0, 'f', 0);
   _globalMapView->message( msg );
 }
 
@@ -2469,8 +2557,13 @@ void Map::slotZoomOut()
 
   zoomFactor *= zoomProgressiveVal[zoomProgressive];
 
+  if( zoomFactor > GeneralConfig::instance()->getMapUpperLimit() )
+    {
+      zoomFactor = GeneralConfig::instance()->getMapUpperLimit();
+    }
+
   scheduleRedraw();
-  QString msg = QString(tr("Map zoom out by factor %1")).arg(zoomFactor, 0, 'f', 1);
+  QString msg = QString(tr("Zoom scale 1:%1")).arg(zoomFactor, 0, 'f', 0);
   _globalMapView->message( msg );
 }
 
@@ -2545,7 +2638,7 @@ void Map::slotSetScale(const double& newScale)
  */
 void Map::__drawDirectionLine(const QPoint& from)
 {
-  if ( ! GeneralConfig::instance()->getMapBearLine() )
+  if ( ! GeneralConfig::instance()->getTargetLineDrawState() )
     {
       return;
     }
@@ -2563,7 +2656,7 @@ void Map::__drawDirectionLine(const QPoint& from)
       QColor col = ReachableList::getReachColor(calculator->getselectedWp()->origP);
 
       // we do take the task course line width
-      qreal penWidth = GeneralConfig::instance()->getTaskCourseLineWidth();
+      qreal penWidth = GeneralConfig::instance()->getTargetLineWidth();
 
       QPainter lineP;
       lineP.begin(&m_pixInformationMap);
@@ -2574,6 +2667,140 @@ void Map::__drawDirectionLine(const QPoint& from)
     }
 }
 
+/**
+ * This function draws a "track line" beginning from the current position in
+ * the moving direction.
+ */
+void Map::__drawTrackLine(const QPoint& from)
+{
+  if( ! GeneralConfig::instance()->getTrackLineDrawState() ||
+      ! calculator || ! calculator->getselectedWp() )
+    {
+      return;
+    }
+
+  // define a radius length
+  int radius = qMax( width(), height() );
+
+  static const double rad = M_PI / 180.;
+
+  // correct angle because the different coordinate systems.
+  int heading = (360 - calculator->getlastHeading()) + 90;
+
+  // Note, that the Cartesian coordinate system must be mirrored at the
+  // the X-axis to get the painter's coordinate system. That means all
+  // angles must be multiplied by -1.
+  double angle = -rad * normalize(heading);
+
+  // Calculate the second point by using polar coordinates.
+  int toX = static_cast<int> (rint(cos(angle) * radius)) + from.x();
+  int toY = static_cast<int> (rint(sin(angle) * radius)) + from.y();
+
+  QPoint to( toX, toY );
+
+  // we do take the task course line width
+  qreal penWidth = GeneralConfig::instance()->getTrackLineWidth();
+  QColor color = GeneralConfig::instance()->getTrackLineColor();
+
+  QPainter lineP;
+  lineP.begin(&m_pixInformationMap);
+  lineP.setClipping(true);
+  lineP.setPen(QPen(color, penWidth, Qt::SolidLine));
+  lineP.drawLine(from, to);
+  lineP.end();
+}
+
+/**
+ * Draws a relative bearing indicator in the upper map area, if the flight
+ * state is cruising or wave.
+ */
+void Map::__drawRelBearingInfo()
+{
+  if( ! GeneralConfig::instance()->getMapShowRelBearingInfo() ||
+      ! calculator || ! calculator->getselectedWp() ||
+        calculator->currentFlightMode() != Calculator::cruising )
+    {
+      return;
+    }
+
+  int heading = calculator->getlastHeading();
+  int bearing = calculator->getlastBearing();
+
+  int relBearing = bearing - heading;
+
+  if( relBearing < -180 )
+    {
+      relBearing += 360;
+    }
+  else if( relBearing > 180 )
+    {
+      relBearing -= 360;
+    }
+
+  if( lastRelBearing != relBearing )
+    {
+      lastRelBearing = relBearing;
+
+      QFont font = this->font();
+
+      int pointSize = 16;
+
+    #ifdef MAEMO
+      pointSize += 4;
+    #endif
+
+      font.setPointSize(pointSize);
+      font.setBold(true);
+
+      QString text = "";
+
+      if( relBearing == 0 )
+        {
+          text = QString("0") + QChar(Qt::Key_degree);
+        }
+      else if( relBearing > 0 )
+        {
+          text = QString("%1").arg(relBearing) + QChar(Qt::Key_degree) + ">>";
+        }
+      else if( relBearing < 0 )
+        {
+          text = QString("<<%1").arg(-relBearing) + QChar(Qt::Key_degree);
+        }
+
+      // calculate text bounding box, if necessary
+      if( relBearingTextBox.isNull() )
+        {
+          QFontMetrics fm(font);
+          relBearingTextBox = fm.boundingRect( "<000°>" );
+        }
+
+      m_pixRelBearingDisplay = QPixmap( relBearingTextBox.width() + 2, relBearingTextBox.height() );
+      m_pixRelBearingDisplay.fill( Qt::black );
+
+      QPainter painter;
+      painter.begin(&m_pixRelBearingDisplay);
+      painter.setFont(font);
+      painter.setPen(QPen(Qt::white));
+      painter.drawText( 0, 0,
+                        relBearingTextBox.width(), relBearingTextBox.height(),
+                        Qt::AlignCenter,
+                        text );
+      painter.end();
+    }
+
+  if( m_pixRelBearingDisplay.isNull() )
+    {
+      return;
+    }
+
+  QPainter painter;
+  painter.begin(&m_pixInformationMap);
+  painter.drawPixmap( width() / 2 - m_pixRelBearingDisplay.width() / 2,
+                      0,
+                      m_pixRelBearingDisplay );
+  painter.end();
+}
+
 /**
  * Check if the new position is near to or inside of an airspace. A warning message
  * will be generated and shown as pop up window and in the status bar.
