@@ -12,7 +12,7 @@
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: map.h 4685 2011-09-21 16:52:31Z axel $
+**   $Id: map.h 4738 2011-12-22 21:51:29Z axel $
 **
 ***********************************************************************/
 
@@ -25,6 +25,8 @@
  *
  * \date 1999-2011
  *
+ * \version $Id: map.h 4738 2011-12-22 21:51:29Z axel $
+ *
  */
 
 #ifndef MAP_H
@@ -41,6 +43,8 @@
 #include <QLabel>
 #include <QEvent>
 #include <QResizeEvent>
+#include <QRect>
+#include <QWheelEvent>
 
 #include "waypoint.h"
 #include "airspace.h"
@@ -84,7 +88,7 @@ public:
                  landmarks,
                  obstacles,
                  hydro,
-                 highways,
+                 motorways,
                  lakes,
                  aeroLayer,
                  airspaces,
@@ -287,6 +291,12 @@ protected:
    */
   virtual void mouseReleaseEvent(QMouseEvent* event);
 
+  /**
+   * Used for zoom action.
+   */
+  virtual void wheelEvent(QWheelEvent *event);
+
+
 private:
   /**
    * set mutex to avoid reentrance
@@ -387,6 +397,11 @@ private:
                     const WGSPoint& origP,      // WGS84 point
                     const bool isLandable );    // is landable?
 
+  /**
+   * Draws the city labels at the map.
+   */
+  void _drawCityLabels( QPixmap& pixmap );
+
   /**
    * Display Info about Airspace items
    */
@@ -435,6 +450,7 @@ private:
    * Draws the Xsymbol on the pixmap
    */
   void __drawX();
+
   /**
    * This function draws a "direction line" on the map if a waypoint
    * has been selected. The QPoint is the projected & mapped
@@ -443,6 +459,18 @@ private:
    */
   void __drawDirectionLine(const QPoint& from);
 
+  /**
+   * This function draws a "track line" beginning from the current position in
+   * the moving direction.
+   */
+  void __drawTrackLine(const QPoint& from);
+
+  /**
+   * Draws a relative bearing indicator in the upper map area, if the flight
+   * state is cruising or wave.
+   */
+  void __drawRelBearingInfo();
+
 #ifdef FLARM
 
   /**
@@ -531,13 +559,17 @@ private: //members
   // new data are are requested by the window system.
   QPixmap m_pixPaintBuffer;
 
+  // Pixmap containing relative bearing display.
+  QPixmap m_pixRelBearingDisplay;
+
   //contains a strip with wind arrows in different directions
   QPixmap windArrow;
 
   int mapRot;
   int curMapRot;
 
-  Waypoint wp;  // currently selected waypoint
+  // currently selected waypoint
+  Waypoint wp;
 
   /**
    * Contains the regions of all visible airspaces. The list is needed to
@@ -562,6 +594,8 @@ private: //members
   unsigned int zoomProgressive;
   float zoomProgressiveVal[8];
 
+  QRect relBearingTextBox;
+
 protected:
   /**
    * Contains the current heading. Used for rotating the map in
@@ -577,6 +611,11 @@ protected:
    */
   int bearing;
 
+  /**
+   * Contains the last calculated relative bearing.
+   */
+  int lastRelBearing;
+
   /**
    * Contains the map mode.
    * northUp: the top of the map is north
@@ -610,6 +649,9 @@ protected:
   QTime _lastVeryNearTime;
   QTime _lastInsideTime;
 
+  /** List of drawn cities. */
+  QList<BaseMapElement *> m_drawnCityList;
+
 public:
 
   static Map *instance;
