@@ -12,7 +12,7 @@
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: mapmatrix.cpp 4221 2010-08-17 19:58:01Z axel $
+ **   $Id: mapmatrix.cpp 4709 2011-11-21 08:18:37Z axel $
  **
  ***********************************************************************/
 
@@ -181,7 +181,7 @@ bool MapMatrix::isVisible( const QRect& itemBorder, int typeID) const
   // ! check for < 10000 is a workaround for a bug other where
   //   that came out after fixing the scale criteria that was always true
   //   before
-  if( typeID == BaseMapElement::Highway ||
+  if( typeID == BaseMapElement::Motorway ||
       typeID == BaseMapElement::Road ||
       typeID == BaseMapElement::Trail ||
       typeID == BaseMapElement::Railway ||
@@ -579,7 +579,7 @@ unsigned int MapMatrix::currentDrawScale() const
  * @returns an indication, if a waypoint can be drawn on the map according to the current
  * scale setting.
  */
-bool MapMatrix::isWaypoint2Draw( wayPoint::Importance importance ) const
+bool MapMatrix::isWaypoint2Draw( Waypoint::Priority importance ) const
 {
   int wpScaleLimit = GeneralConfig::instance()->getWaypointScaleBorder( importance );
 
