@@ -9,7 +9,7 @@
 
     email                : axel@kflog.org
 
-    $Id: mapview.cpp 4592 2011-04-06 18:08:01Z axel $
+    $Id: mapview.cpp 4741 2011-12-23 14:28:14Z axel $
 
  ***************************************************************************/
 
@@ -745,30 +745,41 @@ void MapView::slot_Position(const QPoint& position, const int source)
 /** This slot is called if the status of the GPS changes. */
 void MapView::slot_GPSStatus(GpsNmea::GpsStatus status)
 {
+  QColor color;
+
   switch ( status )
     {
     case GpsNmea::validFix:
       slot_info( tr( "GPS new fix" ) );
+      color = Qt::green;
       break;
     case GpsNmea::noFix:
       slot_info(tr( "GPS fix lost" ) );
+      color = Qt::yellow;
       break;
     default:
       slot_info( tr( "GPS lost" ) );
+      color = Qt::red;
+      break;
     }
 
-  if(status>GpsNmea::notConnected)
+  if( status != GpsNmea::notConnected )
     {
-      _statusGps->setText(tr("GPS"));
+      _statusGps->setText( tr( "GPS" ) );
     }
   else
     {
-      _statusGps->setText(tr("Man"));
-      _heading->setValue("-");
-      _speed->setValue("-");
-      _altitude->setValue("-");
+      _statusGps->setText( tr( "Man" ) );
+      _heading->setValue( "-" );
+      _speed->setValue( "-" );
+      _altitude->setValue( "-" );
     }
 
+  // Set the color background according to the current GPS status.
+  _statusGps->setAutoFillBackground( true );
+  _statusGps->setBackgroundRole( QPalette::Window );
+  _statusGps->setPalette( QPalette( color ) );
+
   // Only show glider symbol if the GPS is connected.
   _theMap->setShowGlider( status == GpsNmea::validFix );
 }
@@ -1040,12 +1051,12 @@ void MapView::slot_OpenMapView()
 
 /** This slot updates the FlightStatus status bar-widget with the
     current logging and flight mode status */
-void MapView::slot_setFlightStatus()
+void MapView::slot_setFlightStatus( Calculator::FlightMode fm )
 {
   QString status="";
 
   // flight mode status
-  switch (calculator->currentFlightMode())
+  switch (fm)
     {
     case Calculator::unknown:
       status += (tr("?","Unknown"));
@@ -1062,9 +1073,6 @@ void MapView::slot_setFlightStatus()
     case Calculator::circlingR:
       status += (tr("R","Circling Right"));
       break;
-    case Calculator::wave:
-      status += (tr("W","Wave"));
-      break;
     }
 
   _statusFlightstatus->setText(status);
@@ -1186,8 +1194,15 @@ void MapView::slot_newAltimeterMode()
   _altitude->setPreUnit( Altitude::getUnitText() );
   _glidepath->setPreUnit( Altitude::getUnitText() );
 
-  // Mode change needs always an altitude display update.
+  // Altimeter unit change needs always an altitude display update.
   slot_Altitude( calculator->getAltimeterAltitude() );
+
+  if( _glidepath->getValue() != "-" )
+    {
+      // Altimeter unit change needs always an arrival display update,
+      // if display shows a real value.
+      slot_GlidePath( calculator->getlastGlidePath() );
+    }
 }
 
 /** Opens the Variometer settings dialog. */
